==> cloud-3.0.0-alpha2.tar.gz: cloud/modules/cloud_service_providers/aws_cloud/src/Form/Ec2/InstanceCreateImageForm.php <==
<?php

namespace Drupal\aws_cloud\Form\Ec2;

use Drupal\Core\Form\FormStateInterface;

/**
 * Create image from an instance.
 */
class InstanceCreateImageForm extends AwsDeleteForm {

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    $entity = $this->entity;

    if ($this->ec2Service->pingMetadataSecurityServer() && $entity->getInstanceId() === $this->ec2Service->getInstanceIdFromMetadata()) {
      $this->messenger()->addWarning($this->t('Attempting to create an image of the instance that is running Cloud Orchestrator.  Cloud Orchestrator will stop responding until the image is created.
      Check "No Reboot" to prevent Cloud Orchestrator from rebooting.'));
    }
    return $this->t('Create an image for instance %name (@instance_id)', [
      '%name' => $entity->label(),
      '@instance_id' => $entity->getInstanceId(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Create Image');
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return '';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    if (!$this->updateEntityBuildForm($this->entity)) {
      return $this->redirectUrl;
    }
    $form['image'] = [
      '#type' => 'details',
      '#title' => $this->t('Image'),
      '#open' => TRUE,
    ];
    $form['image']['image_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Image Name'),
      '#description' => $this->t('A name for the new image.'),
      '#required' => TRUE,
      '#default_value' => $this->entity->label() . '-' . date('Ymd'),
    ];
    $form['image']['no_reboot'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('No Reboot'),
      '#description' => $this->t("By default, Amazon EC2 attempts to shut down and reboot the instance before creating the image. If the 'No Reboot' option is set, Amazon EC2 doesn't shut down the instance before creating the image. When this option is used, file system integrity on the created image can't be guaranteed."),
      '#default_value' => FALSE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\aws_cloud\Entity\Ec2\Instance $entity */
    $entity = $this->entity;
    if (!$this->updateEntitySubmitForm($form_state, $entity)) {
      return;
    }

    $this->ec2Service->setCloudContext($entity->getCloudContext());
    $result = $this->ec2Service->createImage([
      'InstanceId' => $entity->getInstanceId(),
      'Name' => $form_state->getValue('image_name'),
      'NoReboot' => $form_state->getValue('no_reboot') === 0 ? FALSE : TRUE,
    ]);

    if (!empty($entity) && !empty($result['ImageId'])) {

      // Call image update on this particular image.
      $this->ec2Service->updateImages([
        'ImageIds' => [
          $result['ImageId'],
        ],
      ]);

      $this->messenger->addStatus($this->t('The @type %label (%image_id) has been created.', [
        '@type' => $entity->getEntityType()->getSingularLabel(),
        '%label' => $entity->toLink($entity->label())->toString(),
        '%image_id' => $result['ImageId'],
      ]));
      $this->clearCacheValues();

      $form_state->setRedirect('view.aws_cloud_image.list', ['cloud_context' => $entity->getCloudContext()]);
    }
    else {

      $this->processOperationErrorStatus($entity, 'created');
    }
  }

}

==> cloud-4.0.0-alpha2.tar.gz: cloud/modules/cloud_service_providers/aws_cloud/src/Form/Ec2/InstanceCreateImageForm.php <==
<?php

namespace Drupal\aws_cloud\Form\Ec2;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Create image from an instance.
 */
class InstanceCreateImageForm extends AwsDeleteForm {

  /**
   * {@inheritdoc}
   */
  public function getQuestion(): TranslatableMarkup {
    $entity = $this->entity;

    if ($this->ec2Service->isInstanceCloudOrchestrator($this->entity->getInstanceId()) === TRUE) {
      $this->messenger()->addWarning($this->t('Attempting to create an image of the instance that is running Cloud Orchestrator.  Cloud Orchestrator will stop responding until the image is created.
      Check "No Reboot" to prevent Cloud Orchestrator from rebooting.'));
    }
    return $this->t('Create an image for instance %name (@instance_id)', [
      '%name' => $entity->label(),
      '@instance_id' => $entity->getInstanceId(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText(): TranslatableMarkup {
    return $this->t('Create Image');
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription(): string {
    return '';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildForm($form, $form_state);

    if (!$this->updateEntityBuildForm($this->entity)) {
      return $this->redirectUrl;
    }
    $form['image'] = [
      '#type' => 'details',
      '#title' => $this->t('Image'),
      '#open' => TRUE,
    ];
    $form['image']['image_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Image Name'),
      '#description' => $this->t('A name for the new image.'),
      '#required' => TRUE,
      '#default_value' => $this->entity->label() . '-' . date('Ymd'),
    ];
    $form['image']['no_reboot'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('No Reboot'),
      '#description' => $this->t("By default, Amazon EC2 attempts to shut down and reboot the instance before creating the image. If the 'No Reboot' option is set, Amazon EC2 doesn't shut down the instance before creating the image. When this option is used, file system integrity on the created image can't be guaranteed."),
      '#default_value' => FALSE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    /** @var \Drupal\aws_cloud\Entity\Ec2\Instance $entity */
    $entity = $this->entity;
    if (!$this->updateEntitySubmitForm($form_state, $entity)) {
      return;
    }

    $this->ec2Service->setCloudContext($entity->getCloudContext());
    $result = $this->ec2Service->createImage([
      'InstanceId' => $entity->getInstanceId(),
      'Name' => $form_state->getValue('image_name'),
      'NoReboot' => $form_state->getValue('no_reboot') === 0 ? FALSE : TRUE,
    ]);

    if (!empty($entity) && !empty($result['ImageId'])) {

      // Call image update on this particular image.
      $this->ec2Service->updateImages([
        'ImageIds' => [
          $result['ImageId'],
        ],
      ]);

      $this->messenger->addStatus($this->t('The @type %label (%image_id) has been created.', [
        '@type' => $entity->getEntityType()->getSingularLabel(),
        '%label' => $entity->toLink($entity->label())->toString(),
        '%image_id' => $result['ImageId'],
      ]));
      $this->clearCacheValues($entity->getCacheTags());
      $image = $this->loadNewImage($entity->getCloudContext(), $result['ImageId']);
      if (!empty($image)) {
        $this->dispatchSubmitEvent($image);
      }
      $form_state->setRedirect('view.aws_cloud_image.list', ['cloud_context' => $entity->getCloudContext()]);
    }
    else {

      $this->processOperationErrorStatus($entity, 'created');
    }
  }

  /**
   * Helper method to load image entity.
   *
   * @param string $cloud_context
   *   Cloud context.
   * @param string $image_id
   *   Image id to load.
   *
   * @return null|\Drupal\Core\Entity\EntityInterface
   *   NULL if not found else Image entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function loadNewImage($cloud_context, $image_id): ?EntityInterface {
    $entity = NULL;
    $entities = $this->entityTypeManager
      ->getStorage('aws_cloud_image')
      ->loadByProperties([
        'cloud_context' => $cloud_context,
        'image_id' => $image_id,
      ]);

    if (!empty($entities)) {
      $entity = array_shift($entities);
    }
    return $entity;
  }

}
